#!/usr/bin/env python3

__title__ = 'spacedr'
__description__ = 'Spaced Repetition API'
__url__ = 'http://github.com/sttlr/spacedr'
__version__ = '1.0.1'
__author__ = 'sttlr'
__license__ = 'MIT'
__copyright__ = 'Copyright 2018, sttlr'
